/*	$Csoft	    */

/*
 * Copyright (c) 1999, 2000, 2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <techmlm.h>

#ifdef TAMPERWITH

#include <sys/stat.h>
#include <sys/mman.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <glib.h>
#include <unistd.h>

/* Add followup to existing case. */
void 
techmlm_fillcase(struct techmlm *ti)
{
	struct stat sta;

	if (index(ti->cid, '/') || !index(ti->cid, '-')) {
		log(ti, L_WARN, "bad case id: %s", ti->cid);
		return;
	}
	if (ti->cf)
		free(ti->cf);
	ti->cf = g_strdup_printf("%s/%s", ti->ciddir, ti->cid);

	if (stat(ti->cf, &sta) < 0) {
		log(ti, L_WARN, "cannot find case %s (%s)", ti->cid, ti->cf);
	} else {
		char *s;
		s = g_strdup_printf("%s.%i%i%i%i%i", ti->cf, ti->mon,
		    ti->mday, ti->hour, ti->min, ti->sec);
		symlink(ti->af, s);
		free(s);
	}
}


/* New case (no[#] in subject) */
char *
techmlm_newcase(struct techmlm *ti)
{
	char *cid;
	struct stat sta;

	if (ti->casetype)
		cid = g_strdup_printf("%s-%s", ti->casetype, ti->msgid);
	else
		cid = g_strdup_printf("unk-%s", ti->msgid);

	if (ti->cf)
		free(ti->cf);
	ti->cf = g_strdup_printf("%s/%s", ti->ciddir, cid);

	if (stat(ti->cf, &sta) > 0) {
		log(ti, L_ERROR, "existing case id: %s", ti->msgid);
		return (NULL);
	}
	symlink(ti->af, ti->cf);
	return (cid);
}

void 
techmlm_tamperwith(struct techmlm * ti)
{
	size_t fl;
	int f;
	char *s, **l, **lp, *lw;
	char header;

	header = 1;

	f = open(ti->tf, O_RDONLY, 00600);
	if (f < 0)
		return;
	fl = lseek(f, 0, SEEK_END);
	if (fl < 5) {
		close(f);
		return;
	}
	s = mmap(0, fl, PROT_READ, MAP_PRIVATE, f, 0);
	close(f);

	if ((f = open(ti->tf, O_WRONLY)) < 0) {
		munmap(s, fl);
		return;
	}
	l = g_strsplit(s, "\n", 0);
	lp = l;
	while ((lw = *(l++))) {
		if (!strcmp("", lw) && header) {
			header = 0;
			if (ti->email) {
				if (ti->replyto)
					write(f, "Reply-to: ", 10);
				else
					write(f, "Cc: ", 4);
				write(f, ti->email, strlen(ti->email));
				write(f, "\n", 1);
			}
		}
		if (!strncasecmp("x-mailer:", lw, 9) && header) {
			if (strstr(lw, "[#")) {
				if ((ti->casetype = index(lw, '['))
				    && (ti->casetype = strdup(
				        index(ti->casetype, '#') + 1))) {
					if (index(ti->casetype, ']'))
						chopc(ti->casetype, ']');
				}
			}
		}
		if (!strncasecmp("subject:", lw, 8) && header) {
			if (strstr(lw, "[#")) {
				if ((ti->cid = index(lw, '['))
				    && (ti->cid = strdup(
				        index(ti->cid, '#') + 1)))
					if (index(ti->cid, ']'))
						chopc(ti->cid, ']');
				write(f, lw, strlen(lw));
				write(f, "\n", 1);
				techmlm_fillcase(ti);
			} else {
				if ((ti->cid = techmlm_newcase(ti))) {
					write(f, lw, strlen(lw));
					write(f, " [#", 3);
					write(f, ti->cid, strlen(ti->cid));
					write(f, "]\n", 2);
				}
			}
		} else {
			write(f, lw, strlen(lw));
			write(f, "\n", 1);
		}
	}

	close(f);
	g_strfreev(lp);
	munmap(s, fl);
}

#endif
