/*	$Csoft	    */

/*
 * Copyright (c) 1999, 2000, 2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <glib.h>

#include "techmlm-web.h"

static char	*aftercolon_dup(struct tmweb *, char *);

static char *
aftercolon_dup(struct tmweb *twi, char *s)
{
	char *t;
	
	t = strdup(index(s, ':') + 2);

	if (t == NULL) {
		perror("strdup");
		return (NULL);
	}

	*(index(t, '\n')) = '\0';

	return (t);
}

struct tmhead *
parse_header(struct tmweb *twi, char *s, size_t fl, char *msgid)
{
	struct tmhead *twh;
	int i;
	char *sp, *adot, *sch;
	char header;

	sp = s;
	header = 1;
	adot = NULL;
	sch = NULL;

	twh = malloc(sizeof(struct tmhead));
	if (twh == NULL) {
		perror("malloc");
		abort();
	}

	twh->depth = 0;
	twh->from = NULL;
	twh->to = NULL;
	twh->subject = NULL;
	twh->date = NULL;
	twh->msgid = msgid;
	twh->data = NULL;

	if ((adot = strchr(msgid, '.')))
		twh->depth++;

	for (i = 0; i < fl; i++) {
		if (!header || (sp[i] != '\n'))
			continue;

		if (sp[i + 1] == 'f' || sp[i + 1] == 'F')
			twh->from = aftercolon_dup(twi, sp + i + 1);
		else if (sp[i + 1] == 't' || sp[i + 1] == 'T')
			twh->to = aftercolon_dup(twi, sp + i + 1);
		else if (sp[i + 1] == 's' || sp[i + 1] == 'S')
			twh->subject = aftercolon_dup(twi, sp + i + 1);
		else if (sp[i + 1] == 'd' || sp[i + 1] == 'D')
			twh->date = aftercolon_dup(twi, sp + i + 1);

		if ((sp[i] == '\n') && (sp[i + 1] == '\n')) {
			header = 0;
			twh->data = sp + i + 1;
		}
	}

	if (twh->from && (sch = strchr(twh->from, '<'))) {
		*(strchr(sch + 1, '>')) = '\0';
		strcpy(twh->from, sch + 1);
	}
	return (twh);
}

void 
free_header(struct tmhead *twh)
{
	if (twh->from)
		free(twh->from);
	if (twh->to)
		free(twh->to);
	if (twh->subject)
		free(twh->subject);
	if (twh->date)
		free(twh->date);
	if (twh->msgid)
		free(twh->msgid);
	free(twh);
}
