/*	$Csoft	    */

/*
 * Copyright (c) 1999, 2000, 2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>

#include "techmlm.h"

void 
log(struct techmlm *ti, char status, char *s, ...)
{
	va_list ap;
	size_t i;
	FILE *f = fopen(ti->logfile, "a");

	if (!f) {
		perror("fopen");
		return;
	}
	fprintf(f, "[%s] ", ti->msgid);

	switch (status) {
	case L_ERROR:
		fwrite("ERROR: ", 7, 1, f);
		break;
	case L_OK:
		fwrite("OK: ", 4, 1, f);
		break;
	case L_WARN:
		fwrite("WARN: ", 6, 1, f);
		break;
	}

	va_start(ap, s);
	for (i = 0; i < strlen(s); i++) {
		if (s[i] == '%') {
			switch (s[i + 1]) {
			case 's':
				fprintf(f, "%s", va_arg(ap, char *));
				break;
			case 'i':
				fprintf(f, "%i", va_arg(ap, int));
				break;
			case 'c':
				fprintf(f, "%c", va_arg(ap, char));
				break;
			}
			i++;
		} else {
			fwrite(s + i, 1, 1, f);
		}
	}

	fwrite("\n", 1, 1, f);
	fclose(f);
	chmod(ti->logfile, 00600);
	va_end(ap);
}

void 
regetdate(struct techmlm * ti)
{
	struct tm *ltime;
	time_t tt;

	tt = time(NULL);
	ltime = malloc(sizeof(struct tm));

	gmtime_r(&tt, ltime);

	if (ltime) {
		ti->mon = ltime->tm_mon + 1;
		ti->mday = ltime->tm_mday;
		ti->year = ltime->tm_year + 1900;
		ti->sec = ltime->tm_sec;
		ti->min = ltime->tm_min;
		ti->hour = ltime->tm_hour;
	} else {
		log(ti, L_ERROR, "localtime_r() failed");
	}
}

char *
chopc(char *str, char c)
{
	*(index(str, c)) = '\0';
	return (str);
}
